import importlib.util
import sys
from datetime import datetime

import regex as re

TIME_MATCHER = re.compile(
    r".*?"
    r"(?P<hour>2[0-3]|[0-1]\d|\d):"
    r"(?P<minute>[0-5]\d|\d):"
    r"(?P<second>6[0-1]|[0-5]\d|\d)"
    r"\.(?P<microsecond>[0-9]{1,6})"
)

MS_SEARCHER = re.compile(r"\.(?P<microsecond>[0-9]{1,6})")


def _exec_module(spec, module):
    if hasattr(spec.loader, "exec_module"):
        spec.loader.exec_module(module)
    else:
        # This can happen before Python 3.10
        # if spec.loader is a zipimporter and the Python runtime is in a zipfile
        code = spec.loader.get_code(module.__name__)
        exec(code, module.__dict__)


def patch_strptime():
    """Monkey patching _strptime to avoid problems related with non-english
    locale changes on the system.

    For example, if system's locale is set to fr_FR. Parser won't recognize
    any date since all languages are translated to english dates.
    """
    _strptime_spec = importlib.util.find_spec("_strptime")
    _strptime = importlib.util.module_from_spec(_strptime_spec)
    _exec_module(_strptime_spec, _strptime)
    sys.modules["strptime_patched"] = _strptime

    _calendar = importlib.util.module_from_spec(_strptime_spec)
    _exec_module(_strptime_spec, _calendar)
    sys.modules["calendar_patched"] = _calendar

    _strptime._getlang = lambda: ("en_US", "UTF-8")
    _strptime.calendar = _calendar
    _strptime.calendar.day_abbr = ["mon", "tue", "wed", "thu", "fri", "sat", "sun"]
    _strptime.calendar.day_name = [
        "monday",
        "tuesday",
        "wednesday",
        "thursday",
        "friday",
        "saturday",
        "sunday",
    ]
    _strptime.calendar.month_abbr = [
        "",
        "jan",
        "feb",
        "mar",
        "apr",
        "may",
        "jun",
        "jul",
        "aug",
        "sep",
        "oct",
        "nov",
        "dec",
    ]
    _strptime.calendar.month_name = [
        "",
        "january",
        "february",
        "march",
        "april",
        "may",
        "june",
        "july",
        "august",
        "september",
        "october",
        "november",
        "december",
    ]

    return _strptime._strptime_time


__strptime = patch_strptime()


def _prepare_format(date_string: str, og_format: str) -> tuple[str, str]:
    # Adapted from std lib: https://github.com/python/cpython/blob/e34a5e33049ce845de646cf24a498766a2da3586/Lib/_strptime.py#L448
    format = re.sub(r"([\\.^$*+?\(\){}\[\]|])", r"\\\1", og_format)
    format = re.sub(r"\s+", r"\\s+", format)
    format = re.sub(r"'", "['\u02bc]", format)
    year_in_format = False
    day_of_month_in_format = False

    def repl(m: re.Match[str]) -> str:
        format_char = m[1]
        if format_char in ("Y", "y", "G"):
            nonlocal year_in_format
            year_in_format = True
        elif format_char in ("d",):
            nonlocal day_of_month_in_format
            day_of_month_in_format = True

        return ""

    _ = re.sub(r"%[-_0^#]*[0-9]*([OE]?\\?.?)", repl, format)
    if day_of_month_in_format and not year_in_format:
        current_year = datetime.today().year
        return f"{current_year} {date_string}", f"%Y {og_format}"
    return date_string, og_format


def strptime(date_string: str, format: str) -> datetime:
    date_string, format = _prepare_format(date_string, format)
    obj = datetime(*__strptime(date_string, format)[:-3])

    if "%f" in format:
        try:
            match_groups = TIME_MATCHER.match(date_string).groupdict()
            ms = match_groups["microsecond"]
            ms = ms + ((6 - len(ms)) * "0")
            obj = obj.replace(microsecond=int(ms))
        except AttributeError:
            match_groups = MS_SEARCHER.search(date_string).groupdict()
            ms = match_groups["microsecond"]
            ms = ms + ((6 - len(ms)) * "0")
            obj = obj.replace(microsecond=int(ms))

    return obj



info = {
    "name": "hu",
    "date_order": "YMD",
    "january": [
        "jan",
        "január",
        "I"
    ],
    "february": [
        "febr",
        "február",
        "feb",
        "II"
    ],
    "march": [
        "márc",
        "március",
        "már",
        "III"
    ],
    "april": [
        "ápr",
        "április",
        "IV"
    ],
    "may": [
        "máj",
        "május",
        "V"
    ],
    "june": [
        "jún",
        "június",
        "VI"
    ],
    "july": [
        "júl",
        "július",
        "VII"
    ],
    "august": [
        "aug",
        "augusztus",
        "VIII"
    ],
    "september": [
        "szept",
        "szeptember",
        "IX"
    ],
    "october": [
        "okt",
        "október",
        "X"
    ],
    "november": [
        "nov",
        "november",
        "XI"
    ],
    "december": [
        "dec",
        "december",
        "XII"
    ],
    "monday": [
        "h",
        "hétfő"
    ],
    "tuesday": [
        "k",
        "kedd"
    ],
    "wednesday": [
        "sze",
        "szerda"
    ],
    "thursday": [
        "cs",
        "csütörtök"
    ],
    "friday": [
        "p",
        "péntek"
    ],
    "saturday": [
        "szo",
        "szombat"
    ],
    "sunday": [
        "v",
        "vasárnap",
        "vas"
    ],
    "am": [
        "de"
    ],
    "pm": [
        "du"
    ],
    "year": [
        "év",
        "évek",
        "évvel",
        "éve"
    ],
    "month": [
        "hónap",
        "hó",
        "hónapok",
        "hónappal",
        "hónapja"
    ],
    "week": [
        "hét",
        "hetek",
        "héttel",
        "hete"
    ],
    "day": [
        "nap",
        "napok",
        "napja",
        "nappal"
    ],
    "hour": [
        "óra",
        "ó",
        "órák",
        "órája",
        "órával",
        "óráig",
        "órától"
    ],
    "minute": [
        "perc",
        "p",
        "percek",
        "perce",
        "perccel",
        "percig",
        "perctől"
    ],
    "second": [
        "másodperc",
        "mp",
        "másodpercek",
        "másodperce",
        "másodperccel",
        "másodpercig",
        "másodperctől"
    ],
    "relative-type": {
        "0 day ago": [
            "ma"
        ],
        "0 hour ago": [
            "ebben az órában"
        ],
        "0 minute ago": [
            "ebben a percben"
        ],
        "0 month ago": [
            "ez a hónap"
        ],
        "0 second ago": [
            "most"
        ],
        "0 week ago": [
            "ez a hét"
        ],
        "0 year ago": [
            "ez az év"
        ],
        "1 day ago": [
            "tegnap"
        ],
        "1 month ago": [
            "előző hónap"
        ],
        "1 week ago": [
            "előző hét"
        ],
        "1 year ago": [
            "előző év"
        ],
        "in 1 day": [
            "holnap"
        ],
        "in 1 month": [
            "következő hónap"
        ],
        "in 1 week": [
            "következő hét"
        ],
        "in 1 year": [
            "következő év"
        ],
        "2 day ago": [
            "tegnapelőtt"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "(\\d+[.,]?\\d*) napja",
            "(\\d+[.,]?\\d*) nappal ezelőtt"
        ],
        "\\1 hour ago": [
            "(\\d+[.,]?\\d*) órával ezelőtt"
        ],
        "\\1 minute ago": [
            "(\\d+[.,]?\\d*) perccel ezelőtt"
        ],
        "\\1 month ago": [
            "(\\d+[.,]?\\d*) hónappal ezelőtt"
        ],
        "\\1 second ago": [
            "(\\d+[.,]?\\d*) másodperccel ezelőtt"
        ],
        "\\1 week ago": [
            "(\\d+[.,]?\\d*) héttel ezelőtt"
        ],
        "\\1 year ago": [
            "(\\d+[.,]?\\d*) évvel ezelőtt"
        ],
        "in \\1 day": [
            "(\\d+[.,]?\\d*) nap múlva"
        ],
        "in \\1 hour": [
            "(\\d+[.,]?\\d*) óra múlva"
        ],
        "in \\1 minute": [
            "(\\d+[.,]?\\d*) perc múlva"
        ],
        "in \\1 month": [
            "(\\d+[.,]?\\d*) hónap múlva"
        ],
        "in \\1 second": [
            "(\\d+[.,]?\\d*) másodperc múlva"
        ],
        "in \\1 week": [
            "(\\d+[.,]?\\d*) hét múlva"
        ],
        "in \\1 year": [
            "(\\d+[.,]?\\d*) év múlva"
        ]
    },
    "locale_specific": {},
    "skip": [
        "-a",
        "-ai",
        "-akor",
        "-e",
        "-ei",
        "-ekor",
        "-es",
        "-i",
        "-ig",
        "-je",
        "-jei",
        "-ji",
        "-kor",
        "-tól",
        "-től",
        "-áig",
        "-án",
        "-ától",
        "-éig",
        "-én",
        "-étől",
        "-ös",
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ],
    "sentence_splitter_group": 1,
    "ago": [
        "ezelőtt"
    ],
    "in": [
        "múlva"
    ],
    "simplifications": [
        {
            "egy": "1"
        }
    ]
}



#!/usr/bin/env python3

import os
from tempfile import TemporaryFile

from dateparser.data.languages_info import language_locale_dict


def to_string(data):
    result = ""
    language_column_width = 18
    for language in sorted(data):
        result += language
        locales = data[language]
        if locales:
            result += " " * (language_column_width - len(language))
            result += ", ".join("'{}'".format(locale) for locale in sorted(locales))
        result += "\n"
    return result


def main():
    readme_path = os.path.join(
        os.path.dirname(__file__), "..", "docs", "supported_locales.rst"
    )
    new_data = to_string(language_locale_dict)
    temporary_file = TemporaryFile("w+")
    with open(readme_path) as readme_file:
        delimiter = "============    ================================================================\n"
        delimiters_seen = 0
        is_inside_table = False
        for line in readme_file:
            if line == delimiter:
                delimiters_seen += 1
                is_inside_table = delimiters_seen == 2
            elif is_inside_table:
                continue
            temporary_file.write(line)
            if is_inside_table:
                temporary_file.write(new_data)
    temporary_file.seek(0)
    with open(readme_path, "w") as readme_file:
        readme_file.write(temporary_file.read())
    temporary_file.close()


if __name__ == "__main__":
    main()



info = {
    "name": "ksf",
    "date_order": "DMY",
    "january": [
        "ŋ1",
        "ŋwíí a ntɔ́ntɔ"
    ],
    "february": [
        "ŋ2",
        "ŋwíí akǝ bɛ́ɛ"
    ],
    "march": [
        "ŋ3",
        "ŋwíí akǝ ráá"
    ],
    "april": [
        "ŋ4",
        "ŋwíí akǝ nin"
    ],
    "may": [
        "ŋ5",
        "ŋwíí akǝ táan"
    ],
    "june": [
        "ŋ6",
        "ŋwíí akǝ táafɔk"
    ],
    "july": [
        "ŋ7",
        "ŋwíí akǝ táabɛɛ"
    ],
    "august": [
        "ŋ8",
        "ŋwíí akǝ táaraa"
    ],
    "september": [
        "ŋ9",
        "ŋwíí akǝ táanin"
    ],
    "october": [
        "ŋ10",
        "ŋwíí akǝ ntɛk"
    ],
    "november": [
        "ŋ11",
        "ŋwíí akǝ ntɛk di bɔ́k"
    ],
    "december": [
        "ŋ12",
        "ŋwíí akǝ ntɛk di bɛ́ɛ"
    ],
    "monday": [
        "lǝn",
        "lǝndí"
    ],
    "tuesday": [
        "maa",
        "maadí"
    ],
    "wednesday": [
        "mɛk",
        "mɛkrɛdí"
    ],
    "thursday": [
        "jǝǝ",
        "jǝǝdí"
    ],
    "friday": [
        "júm",
        "júmbá"
    ],
    "saturday": [
        "sam",
        "samdí"
    ],
    "sunday": [
        "sɔ́n",
        "sɔ́ndǝ"
    ],
    "am": [
        "sárúwá"
    ],
    "pm": [
        "cɛɛ́nko"
    ],
    "year": [
        "bǝk"
    ],
    "month": [
        "ŋwíí"
    ],
    "week": [
        "sɔ́ndǝ"
    ],
    "day": [
        "ŋwós"
    ],
    "hour": [
        "cámɛɛn"
    ],
    "minute": [
        "mǝnít"
    ],
    "second": [
        "háu"
    ],
    "relative-type": {
        "0 day ago": [
            "gɛ́ɛnǝ"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "rinkɔɔ́"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "ridúrǝ́"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}
